#! /usr/bin/env python
# standard library modules
import sys
from Queue import Queue
from Queue import Empty
from Queue import Full
import time as _time
import traceback


class PeekableQueue(Queue):
	"""Extension of Python's Queue class to add a peek method.
	
	
	"""
	def __init__(self, maxsize=0):
		# Call superclass constructor.
		Queue.__init__(self, maxsize=maxsize)
	
	def	peek(self, block=True, timeout=None):
		"""Peek at (return) the first item from the queue.
	
		If optional	args 'block' is	true and 'timeout' is None (the	default),
		block if necessary until an	item is	available. If 'timeout'	is
		a positive number, it blocks at	most 'timeout' seconds and raises
		the	Empty exception	if no item was available within	that time.
		Otherwise ('block' is false), return an	item if	one	is immediately
		available, else	raise the Empty	exception ('timeout' is	ignored
		in that	case).
		"""
		self.not_empty.acquire()
		try:
			if not block:
				if self._empty():
					raise Empty
			elif timeout is	None:
				while self._empty():
					self.not_empty.wait()
			else:
				if timeout < 0:
					raise ValueError("'timeout'	must be	a positive number")
				endtime	= _time.time() + timeout
				while self._empty():
					remaining =	endtime	- _time.time()
					if remaining <=	0.0:
						raise Empty
					self.not_empty.wait(remaining)
			item = self._peek()
			self.not_full.notify()
			return item
		finally:
			self.not_empty.release()


	#Peek at leftmost (front) item in the queue. 
	#(See the example in the Python docs for the deque datatype in the collection module -- section 5.12)
	def _peek(self):
		return self.queue[0]	